<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\ZmartLog as ZmarLogService;
use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DTO;
use Util\Utilities;
use Service\Traits\TResponse;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\FilterCriteria;

class ZmartLog extends Crud
{   
    use TResponse;
    protected $dataSourceName = 'zmartLog';
    protected $DeleteCascadeService;
    protected $zmartLogService;

    /** @var \Service\Manager\Mongo\Manager */
    protected $organizationMongoManager;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, ZmarLogService $zmartLogService)
    {   $this->zmartLogService = $zmartLogService;
        $this->organizationMongoManager = $organizationMongoManager;
        $this->DeleteCascadeService = $deleteCascadeService;
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        $dataSet = $this->zmartLogService->findNext($params);
        $zmartLogService = Utilities::parseDataSourceZeedhiNext($dataSet);
        $this->sendResponse(Utilities::paginationAndOrderINPhpNext($zmartLogService, $params), 200);
    }

     public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $sort = $this->mongoManager->processSort($filterData);
        if(empty($sort)){
            $sort = array("DATA_UPDATE" =>-1 );
        }
        $dataSet = $this->mongoManager->findByPagination($filterData,$sort);
        $response->addDataSet($dataSet);
        $dataSet = $response->getDataSets()[0];
        Utilities::datasetMongoToZeedhi($response, $dataSet,$this->dataSourceName);

    }



}
