<?php

namespace Controller;

use Service\ZmartOperator as ZmartOperatorService;
use Zeedhi\Framework\DataSource\DataSet;
use Service\Traits\TResponse;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Util\Utilities;
use Teknisa\Libs\Util\OAuthCustom;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Util\InstanceProvider;
use Zeedhi\Framework\DependencyInjection\InstanceManager;


class ZmartOperator{

    use TResponse;
    /** @var ZmartOperatorService $operatorService */
    protected $operatorService;
    protected $dataSourceName = "zmartOperator";

    public function __construct(ZmartOperatorService $operatorService) {
        $this->operatorService = $operatorService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        try {
            $operators = $this->operatorService->find($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('zmartOperator', $operators));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function findNext(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $filter = new ZmartFilterCriteria($this->dataSourceName, array());
        try {
            $operators = $this->operatorService->find($filter, $params['page'], $params['limit']);
            $this->sendResponse(Utilities::parseDataSourceZeedhiNext($operators), 200);
        } catch (\Exception $e) {
            $this->sendResponse(['error' => $e->getMessage(), 'code' => $e->getCode()], 500);
        }
    }

    public function saveOperatorEmail(DTO\Request $request, DTO\Response $response) {
        $row = $request->getParameters();
        $cdoperador = $row['cdoperador'];
        $dsemailoper = $row['dsemailoper'];
        $organizationId = $row['nrorg'];
        $cdOperadorAtualizacao = $row['cdoperultatu'];
        $nrorgAtualizacao = $row['nrorgultatu'];
        try {
            $operators = $this->operatorService->saveOperatorEmail($cdoperador,$dsemailoper,$organizationId, $cdOperadorAtualizacao, $nrorgAtualizacao);
            $response->addDataSet(new DataSet('saveOperatorEmail', $operators));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function findByNrorg(DTO\Request $request, DTO\Response $response) {
        $row = $request->getParameters();
        $conditions = $row['conditions'];
        $page = $row['page'];
        $pageSize = $row['pageSize'];
        $orderBy = $row['orderBy'];
        $organizationId = $row["NRORG"];
        try {
            $operators = $this->operatorService->findByNrorg($organizationId, $conditions, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSet('zmartOperator', $operators));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }


    public function requestToken(Request $request, Response $response){
        try{
            $kernel = InstanceManager::getInstance()->getService('kernel');
            $oauth = InstanceManager::getInstance()->getService('oAuthImpl');
            $httpRequest = $kernel->getHttpRequest();
            $clientId = $httpRequest->get('CLIENT_ID');
            $clientSecret = $httpRequest->get('CLIENT_SECRET');
            $token = $oauth->grantAccessToken($clientId,$clientSecret,
                array('sessionId' => InstanceProvider::getSession()->getId()));
            $response->addDataSet(new DataSet('token', array($token)));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message(\Util\Messages::ERR_SPONSOR_NOT_SAVED . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }


}