<?php

namespace Service;

use Util\RijndaelCrypt;
use Util\Utilities;
use Util\InstanceProvider;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Controller\Exception;
use Service\SocketClient;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class DataSourceService {

    protected $connectionService;

    protected $defaultColumnsService;

    protected $defaultRelationsService;

    public function __construct(ConnectionService $connectionService, DefaultColumns $defaultColumnsService, DefaultRelations $defaultRelationsService) {
        $this->connectionService = $connectionService;
        $this->defaultColumnsService = $defaultColumnsService;
        $this->defaultRelationsService = $defaultRelationsService;
    }

    public function insertNewDefaultColumns(array &$columns) {
        foreach($columns as &$column) {
            if(isset($column['_altered']) && $column['_altered']) {
                $this->verifyRepeatedNames($columns,[$column]);
                $this->defaultColumnsService->insertNewDefault($column);
            }
        }
        foreach($columns as &$column) {
            unset($column['_altered']);
        }
    }

    public function insertNewDefaultRelations(array $relations) {
        foreach($relations as $relation) {
            $this->defaultRelationsService->insertNewDefault($relation);
        }
    }

    public function getMergedColumns ($row) {
        $columnsName = $this->getColumnsName($row);
        return $this->mergeColumns($row, $columnsName);
    }

    public function getMergedColumnsFromWebSocket (SocketClient $webSocketClient, $query, $row, $organizationId) {
        $connectionParams = $this->connectionService->getConnectionParams(isset($row['connection']) ? $row['connection'] : null);

        $newConnectionParams = array(
			"userNameSsl"	  => isset($connectionParams["usernameSsl"])    ? $connectionParams["usernameSsl"]    : null,
			"passwordSsl"	  => isset($connectionParams["passwordSsl"])    ? $connectionParams["passwordSsl"]    : null,
			"dbUrlSsl" 	  => isset($connectionParams["dbUrlSsl"])	     ? $connectionParams["dbUrlSsl"]       : null,
			"dialect"	  => isset($connectionParams["dialect"])     ? $connectionParams["dialect"]     : null,
            "dbUrlSsl"    => isset($connectionParams['dbUrlSsl'])    ? $connectionParams['dbUrlSsl']    :null,
            "passwordSsl" => isset($connectionParams['passwordSsl']) ? $connectionParams['passwordSsl'] :null,
            "usernameSsl" => isset($connectionParams['usernameSsl']) ? $connectionParams['usernameSsl'] :null,
            "driverClass" => isset($connectionParams["driverClass"]) ? $connectionParams["driverClass"] : null
		);

        $saveQuery = "SELECT * FROM (".$query.") A WHERE 1=2";
        if($newConnectionParams["driverClass"] == "Spreadsheet"){
            $saveQuery = $query;
        }
        $clientRoom = $this->connectionService->getClientRoom($organizationId, $row);
        $response = $webSocketClient->getColumnsMetaData($saveQuery, $clientRoom, null, $newConnectionParams);
        $response = empty($response)? array() : $response[0];

        if (isset($response['event']) && $response['event'] === SocketClient::NO_CLIENTS_CONNECTED) {
            throw Exception::connectFail();
        } else if (isset($response['hasError']) && $response['hasError'] === true) {
            throw Exception::executeQueryError($query, new \Exception($response['error']));
        }

        $columns = !empty($response['data']) && !empty($response['data']['response'])? $response['data']['response'] : array();

        return $this->mergeColumns($row, $this->mapColumnsResult($columns));
    }

    protected function mapColumnsResult($columns) {
        $mappedResult = array();

        foreach ($columns as $column) {
            $mappedResult[] = $column['name'];
        }

        return $mappedResult;
    }

    protected function castObjectToArray(&$object){
        $object = (array)$object;
        foreach($object as &$value){
            if(gettype($value) == "object"){
                $this->castObjectToArray($value);
            }
        }
    }

    public function changeDataSourcesWithOldId($row,$indexes=null){
        $replaceRow = array(
            "query" => $row["query"],
            "columns" => $row["columns"],
            "relations" => $row["relations"]
        );
        $mongoDbName = InstanceManager::getInstance()->getParameter('mongoDBNamePrefix');
        $mongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $mongoAdmin=$mongoManager->buildManager("admin");
        $databases = $mongoAdmin->command(array("listDatabases"=>1));
        $databaseNames = array_column((array)$databases[0]["databases"],"name");
        $nrorgs = [];
        foreach($databaseNames as $name){
            if($name != $mongoDbName && $name != $mongoDbName."5" && substr($name,0,strlen($mongoDbName)) === $mongoDbName){
                $mongo = $mongoManager->buildManager($name);
                $dataSource = $mongo->findBy(new FilterCriteria("dataSource",
                    array(array("columnName"=>"oldId", "value" => $row["_id"], "operator" => "="))
                ))->getRows();
                if(!empty($dataSource) && !empty($dataSource[0])){
                    $newId = $dataSource[0]["_id"];
                    if(isset($dataSource[0]["modified"]) && !$dataSource[0]["modified"]){
                        if(!empty($row["confirm"])){
                            if(empty($indexes)){
                                $newDataSource = array_merge($dataSource[0],$replaceRow);
                                $mongo->persist(new DataSet('dataSource', [$newDataSource]));
                                $this->castObjectToArray($dataSource[0]["columns"]);
                                $this->castObjectToArray($dataSource[0]["relations"]);
                                Utilities::setLog($dataSource, [$newDataSource], "dataSource", $newId,"",$mongo);
                            }else{
                                $columns = $indexes["COLUMN_"];
                                $orders = $indexes["ORDER_"];
                                $name = $indexes["name"];
                                $keys = [];
                                foreach($columns as $i => $column){
                                    if($column){
                                        $keys[$column] = (int)$orders[$i];
                                    }
                                }
                                $options = array("name" => $name);
                                $mongo->createIndex($newId, $keys, $options);
                                Utilities::setLog(null, [$indexes], 'index',$newId,"",$mongo);
                            }
                            
                        }else{
                            $nrorgs[] = str_replace($mongoDbName,"",$name);
                        }
                    }
                }
            }
        }
        if(!empty($nrorgs)){
            $query = "SELECT NRORG,NMORGANIZACAO FROM ORGANIZACAO WHERE NRORG IN (".implode(",",$nrorgs).")";
            $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection(5,true);
            $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
            $rows = $entityManager->getConnection()->fetchAll($query);
            if(!empty($rows) && !empty($rows[0]) && !empty($rows[0]["NMORGANIZACAO"])){
                $orgNames = Utilities::getArrayFromAssociativeArray($rows,"NMORGANIZACAO");
                $orgNrorgs = Utilities::getArrayFromAssociativeArray($rows,"NRORG");
                return array_combine($orgNrorgs,$orgNames);
            }
        }
        return null;
    }

    public function mergeDefaultRelations($row,$structureTypesOldIdList=[]) {
        $relatedStructure = isset($row['relatedStructuresType']) ? $row['relatedStructuresType'] : array();
        if(!empty($structureTypesOldIdList)){
            foreach($relatedStructure as &$structure){
                if(!empty($structureTypesOldIdList[$structure["structureType"]])){
                    $structure["oldStructureType"] = $structure["structureType"];
                    $structure["structureType"] = $structureTypesOldIdList[$structure["structureType"]];
                }
            }
        }
        $defaultRelations = $this->defaultRelationsService->filterFind(new FilterCriteria("defaultRelations"))->getRows();
        foreach($defaultRelations as $defaultRelation) {
            $index = $this->getStructureRelationIndex($relatedStructure, $defaultRelation['structureType']);
            if($index === false) {
                $cleanedRelation = $this->removeNonexistentColumnsFromRelation($row, $defaultRelation);
                $relatedStructure[] = $cleanedRelation;
            }
        }
        return $relatedStructure;
    }

    public function removeNonexistentColumnsFromRelation($row, $relatedStructure) {
        foreach($relatedStructure['relations'] as $index => $relation) {
            $indexedColumns = $this->indexColumnsByName($row);
            if(!$this->hasColumnIndexedByName($indexedColumns, $relation['dataSourceColumn'])) {
                unset($relatedStructure['relations'][$index]);
            }
        }
        unset($relatedStructure['_id']);
        return $relatedStructure;
    }

    public function getStructureRelationIndex($relations, $structureTypeId) {
        $index = false;
        foreach($relations as $indexValue => $relation) {
            if($relation['structureType'] == $structureTypeId) {
                $index = $indexValue;
            }
        }
        return $index;
    }

    /**
     * @param $row
     * @return array
     * @throws \Exception
     */
    public function getColumnsName ($row) {
        $connectionId = isset($row['connection']) ? $row['connection'] : null;
        $connection = $this->connectionService->getCustomConnection($connectionId);
        $columnsQueryService = new ColumnsQueryService($connection);
        return $columnsQueryService->getColumnsOfAQuery($row['query']);
    }

    public function mergeColumns ($row, $columnsName) {
        $indexedColumns = $this->indexColumnsByName($row);
        $externalColumns = $indexedColumns;
        $newColumns = array();
        foreach($columnsName as $columnName) {
            $newColumns[] = $this->getColumn($indexedColumns, $columnName);
        }
        if (isset($row['add'])) {

            $externalColumns = $this->getExternalColumns($externalColumns, $row["queryAdd"]);
        }

        if (isset($row['edit'])) {

            $externalColumns = $this->getExternalColumns($externalColumns, $row["query"]);
        }
        
        $this->verifyRepeatedNames($newColumns,$externalColumns);
        $newColumns = array_merge($newColumns, $externalColumns);

        return $newColumns;
    }

    protected function verifyRepeatedNames($newColumns, $externalColumns){
        foreach($newColumns as $newColumn){
            foreach($externalColumns as $externalColumn){
                if(( (empty($newColumn["_altered"]) && empty($externalColumn["isGenerated"])) || (!empty($newColumn["isGenerated"]) && empty($externalColumn["isGenerated"]))) && $externalColumn["name"] == $newColumn["name"]){
                    throw new \Exception($externalColumn["name"],1);
                }
            }
        }
    }

    protected function getExternalColumns(array &$externalColumns, $query = null) {
        $columns = array();
        foreach ($externalColumns as $column) {
            if(!Utilities::isColumnGenerated($column)){
                $column['isGenerated'] = false;
                unset($column['isExternal']);
                unset($column['function']);
                $columns[] = $column;
            }
        }
        return $columns;
    }
    

    protected function getColumn(array &$indexedColumns, $columnName) {
        if($this->hasColumnIndexedByName($indexedColumns, $columnName)) {
            $column = $indexedColumns[$columnName];
            $column['isGenerated'] = true;
            unset($column['isExternal']);
            unset($column['expression']);
            unset($indexedColumns[$columnName]);
        }
        else {
            $column = $this->applyDefaultOrFactory($columnName);
        }
        return $column;
    }

    protected function applyDefaultOrFactory($columnName) {
        $defaultColumnDataSet = $this->defaultColumnsService->findByName($columnName);
        $defaultColumnRows = $defaultColumnDataSet->getRows();
        if(!empty($defaultColumnRows)) {
            $column = $this->applyDefault($defaultColumnRows[0]);
        }
        else {
            $column = $this->factoryColumn($columnName);
        }
        return $column;
    }

    protected function applyDefault($defaultColumn) {
        $defaultColumn['idGenerated'] = true;
        unset($defaultColumn['_id']);
        return $defaultColumn;
    }

    protected function factoryColumn ($columnName) {
        return array(
            "name" => $columnName,
            "label" => $columnName,
            "isGenerated" => true
        );
    }

    protected function hasColumnIndexedByName ($indexedColumnsArray, $name) {
        return isset($indexedColumnsArray[$name]);
    }

    protected function indexColumnsByName ($row) {
        $columns = isset($row['columns']) ? $row['columns'] : array();
        $indexedColumns = array();
        foreach($columns as $column) {
            $indexedColumns[$column['name']] = $column;
        }
        return $indexedColumns;
    }
}