<?php

namespace Service;


use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;

class NotificationFail {



    const collectionNotificationFailEmail = 'NotificationFailEmail';
    const PAGE_SIZE = 9999999999;
    /**
     * @var Manager
     */
    private $mongoManagerZmart;

    Public function saveConfigEmail($saas,$host,$local){
        $saas = is_array($saas) ? $saas : explode(',', $saas);
        $host = is_array($host) ? $host : explode(',', $host);
        $local = is_array($local) ? $local : explode(',', $local);
        $row = array(
            'EMAIL_SAAS' => $saas,
            'EMAIL_HOST' => $host,
            'EMAIL_LOCAL' => $local,
        );
        $mongoManagerZmart = LogImportation::buildManager();
        $dataSet = new DataSet(self::collectionNotificationFailEmail, [$row]);
        $mongoManagerZmart->dropCollection(self::collectionNotificationFailEmail);
        $mongoManagerZmart->persist($dataSet);
    }

    public function findFail($conditions){
        $this->mongoManagerZmart = $this->buildManager();
        Utilities::convertCreateFilterDateToMongoDate($conditions,'LAST_IMPORTATION');
        Utilities::convertValueFilerLt($conditions);
        return $this->simpleFind($conditions,self::collectionNotificationFailEmail,'dateLog',FilterCriteria::ORDER_DESC);
    }

    private function simpleFind($conditions,$collection = self::collectionLogImportation,$orderBy = null,$direction = null): array
    {
        $filterCriteria = new ZmartFilterCriteria($collection, $conditions);
        $filterCriteria->setPageSize(self::PAGE_SIZE);
        $filterCriteria->setPage(1);
        if($orderBy && $direction){
            $filterCriteria->addOrderBy($orderBy,$direction);
        }
        $dataSet = $this->mongoManagerZmart->simpleFindPagination($filterCriteria);
        foreach ($dataSet as &$row) {
            foreach ($row as &$data) {
                if ($data instanceof \MongoDB\BSON\UTCDateTime) {
                    $data = $data->toDateTime()->format('d/m/Y');
                } else if ($data instanceof \MongoDB\BSON\ObjectId) {
                    $data = (string)$data;
                }
            }
        }
        return $dataSet;
    }

    /** Method to generate a new mongo manager **/
    public static function buildManager($nrorg = null){
        $im = InstanceManager::getInstance();
        $server = $im->getParameter('mongoHost');
        $port =  $im->getParameter('mongoPort');
        $mongoOptions =  $im->getParameter('mongoOptions');
        $mongoDBNamePrefix =  $im->getParameter('mongoDBNamePrefix');
        $mongoDBName =  $im->getParameter('mongoDBName');

        $mongo = new \Zmart\Mongo\Mongo($server, $port, $mongoDBNamePrefix, $mongoOptions);
        $newManager = new Manager($mongo, $im->getService("nameProvider"), $im->getService("mongoQueryBuilder"));
        if($nrorg){
            $mongo->setDbByName($nrorg);
        }else{
            $mongo->setDbByNameBase($mongoDBName);
        }
        return $newManager;
    }

}
