import { Datasource, Metadata, Http, FormatterParserProvider, Loader, IEventParam, IDictionary } from '@zeedhi/core';
import { MasterDetail, Modal, ModalService, Grid, Form, CodeEditor, LoadingService, AlertService, DialogService, Tabs, SelectMultiple, Select } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';

Metadata.get('metadata/ManagementStructures/Modal/managementStructuresAddModal.json');
Metadata.get('metadata/ManagementStructures/Modal/managementStructuresAddResponsibleUserModal.json');

FormatterParserProvider.registerFormatter('column_usersResponsible', ({ row }: { row: any }) => {
    const names = Loader.getInstance('ManagementStructuresController');
    const namesVet = [];
    if (row.usersResponsible && row.usersResponsible !== '') {
      for (let i = 0; i <= row.usersResponsible.length; i += 1) {
          if (names.requestNames[row.usersResponsible[i]]) {
            namesVet.push(names.requestNames[row.usersResponsible[i]]);
          }
      }
    }
    return `${namesVet.join(', ')}`;
});

FormatterParserProvider.registerFormatter('column_accessProfiles', ({ row }: { row: any }) => {
    const accessProfiles = Loader.getInstance('ManagementStructuresController');
    const accessProfilesVet = [];
    if (row.accessProfiles && row.accessProfiles !== '') {
      for (let i = 0; i <= row.accessProfiles.length; i += 1) {
          if (accessProfiles.requestAccessProfiles[row.accessProfiles[i]]) {
            accessProfilesVet.push(accessProfiles.requestAccessProfiles[row.accessProfiles[i]]);
          }
      }
    }
    return `${accessProfilesVet.join(', ')}`;
});

export class ManagementStructuresController {

  private modal!: Modal;
  private addStructureTypeModal!: Modal;
  private importLoadSeveralStructuresFromQuery!: Modal;
  private managementStructuresAddModal!: Modal;
  private managementStructuresCurrentRow: any = {};
  public value: any = false;
  public disabled: any = true;
  public row: any = '';
  public nameManagementStructures: any = '';
  public usersNamesManagementStructures: any = [];
  public requestNames: any = '';
  public requestAccessProfiles: any = '';
  public name: string = '';
  public showAnotherDetail = false;
  public responsibleUserValue: any = [];
  public structMessages: any = '';
  public StructureDados: any = '';

  get anotherMasterIsVisible() {
    return !this.showAnotherDetail;
  }

  private async deleteAndUpdate(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }

  public showAnotherMaster() {
    this.showAnotherDetail = false;
  }

  public rowClickHierarchyType({ event, component}: any) {
    const managementStructuresTab2grid = Metadata.getInstance<TekGrid>('managementStructuresTab2grid');
    const hierarchyTypeDetailTabs = Metadata.getInstance<Tabs>('hierarchyTypeDetailTabs');
    const hierarchyTypeTabForm = Metadata.getInstance<Form>('hierarchyTypeFormTab1');
    hierarchyTypeTabForm.children.forEach(function (formTabHierarchyType) {
      formTabHierarchyType.disabled = false;
    });
    Metadata.getInstance<Form>('hierarchyTypeFormTab1').value = component.datasource.currentRow;
    if (component.datasource.currentRow.isActive === 'Y') {
      Metadata.getInstance<Form>('hierarchyTypeFormTab1').value.isActive = true;
    }
    managementStructuresTab2grid.isVisible = false;
    hierarchyTypeDetailTabs.isVisible = true;
    event.preventDefault();
  }

  public async rowClick({ component, event, row }: any) {
    event.preventDefault();
    LoadingService.show();
    await this.loadFieldKeysFromQueryEdit();
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const headerCardFormStructuresManagement = Metadata.getInstance<Form>('headerCardFormStructuresManagement');
    const managementStructuresMasterDetailTab1FormResult = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');

    const keyAreaTextResult = Metadata.getInstance<SelectMultiple>('keyAreaTextResult');
    const descriptionFieldtextResult = Metadata.getInstance<Select>('descriptionFieldtextResult');
    const conexionTextResult = Metadata.getInstance<Select>('connection');

    if (row.isActive === "Y") {
      row.isActive = true;
    } else {
      row.isActive = false;
    }

    managementStructuresMasterDetailTab1FormResult.value.nameTextResult = row.name;
    managementStructuresMasterDetailTab1FormResult.value.activeCheckBox = row.isActive;
    managementStructuresMasterDetailTab1FormResult.value.queryResult = row.query;
    conexionTextResult.setValue(row.connection);
    descriptionFieldtextResult.setValue(row.descriptionField);
    keyAreaTextResult.setValue(row.fieldKey);

    headerCardFormStructuresManagement.isVisible = true;
    managementStructures.isVisible = true; // Aparece o masterDetail
    managementStructuresTab1grid.isVisible = false; // some com grid
    managementStructuresMasterDetailTab1FormResult.isVisible = true; // Aparece form com o MasterDetail
    this.name = row.name;
    this.showAnotherDetail = true;
    /* managementStructuresTab1grid.datasource.setFilter(
      {
        "nome": "type",
        "operador": "=",
        "value": row._id
      }
      ); */
    // await managementStructuresTab1grid.datasource.get();
    // await this.getManagementStructuresValues(row);
    LoadingService.hide();
  }

  public loadSeveralStructuresFromQueryCloseModal() {
    this.importLoadSeveralStructuresFromQuery?.destroy();
    this.importLoadSeveralStructuresFromQuery.hide();
  }

  public async loadSeveralStructuresFromQuery({component}: any) {
         let structMessage = "";
         this.structMessages = "";
         this.StructureDados = "";
         const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
         const data = managementStructuresTab1grid.datasource.data;
         LoadingService.show();
         const structure = await Http.post("/structure/getManyStructuresFromQuery", data);
         this.StructureDados = structure;
         LoadingService.hide();
         if (structure.data.length === 0) {
           DialogService.show({
                name: "base-edit-dialog",
                type: "info",
                title: "Atenção",
                text: "Não há estruturas para serem adicionadas.",
                cssStyle: "text-align: center"
            });
         } else {
            structure.data.forEach(function (str: any) {
              structMessage += str.typeName + " - " + str.name + "\n";
            });
            this.structMessages = structMessage;
            this.importLoadSeveralStructuresFromQuery = await ModalService.createFromJson('ManagementStructures/Modal/loadSeveralStructuresFromQuery', true);
            this.importLoadSeveralStructuresFromQuery.show();
         }
  }

  public loadSeveralStructuresFromQueryModal() {
    const loadSeveralStructuresFromQueryForm = Metadata.getInstance<Form>('loadSeveralStructuresFromQueryForm');
    loadSeveralStructuresFromQueryForm.value.textareaResult = this.structMessages;
  }

  public async saveImportLoadSeveralStruct() {
    const dados = {
            'dataset': this.StructureDados.data,
            'requestType': 'DataSet'
        };
    LoadingService.show();
    await Http.post("/structure/save", dados);
    LoadingService.hide();
    this.loadSeveralStructuresFromQueryCloseModal();
    CustomAlertService.saveSuccessAlert();
  }

  public async loadFieldKeysFromQueryAdd() {
    const formModal = Metadata.getInstance<Form>('structureTypeForm');
    formModal.value.add = true;

    if (formModal.value.queryAdd) {
      LoadingService.show();
      const data = await Http.post("/structureType/getFieldKeysFromQuery", formModal.value);
      const fieldKey = Metadata.getInstance<SelectMultiple>('fieldKey');
      const descriptionField = Metadata.getInstance<Select>('descriptionField');
      fieldKey.datasource.updateData(data.data);// atribuindo valores para dentro do datasource(data)
      descriptionField.datasource.updateData(data.data);
      await Promise.all([fieldKey, descriptionField]);
      LoadingService.hide();
    }
  }

  public async loadFieldKeysFromQueryEdit() {
    const form = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    form.value.edit = true;

    if (form.value.query) {
      LoadingService.show();
      const data = await Http.post("/structureType/getFieldKeysFromQuery", form.value);
      const keyAreaTextResult = Metadata.getInstance<SelectMultiple>('keyAreaTextResult');
      const descriptionFieldtextResult = Metadata.getInstance<Select>('descriptionFieldtextResult');
      keyAreaTextResult.datasource.updateData(data.data);
      descriptionFieldtextResult.datasource.updateData(data.data);
      await Promise.all([keyAreaTextResult, descriptionFieldtextResult]);
      LoadingService.hide();
    }
  }

  /* public async createManagementStructuresAddModal() {
    if (!this.managementStructuresAddModal) {
       this.managementStructuresAddModal = await ModalService.createFromJson('ManagementStructures/Modal/managementStructuresAddModal', true);
    }
  }

  public openManagementStructuresAddModal() {
    this.managementStructuresAddModal.show();
  }

  public closeManagementStructuresAddModal() {
    this.managementStructuresAddModal.hide();
  } */

  public async getManagementStructuresValues(row: any) {
    this.row = row;
    this.nameManagementStructures = row.name;
    const managementStructures = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    const structureTypeUri = '/structureType';
    const structureTypeRequest = await Http.get(structureTypeUri);
    for (let i = 0; i <= structureTypeRequest.data.data.length; i += 1) {
      if (row.name === structureTypeRequest.data.data[i].name) {
         managementStructures.value.conexionTextResult = structureTypeRequest.data.data[i].connectionName;
         if (!structureTypeRequest.data.data[i].fieldKey) {
           structureTypeRequest.data.data[i].fieldKey = '';
         }
         managementStructures.value.keyAreaTextResult = structureTypeRequest.data.data[i].fieldKey;
         managementStructures.value.nameTextResult = structureTypeRequest.data.data[i].name;
         managementStructures.value.descriptionFieldtextResult = structureTypeRequest.data.data[i].descriptionField;
         if (structureTypeRequest.data.data[i].isActive === 'Y') {
            this.value = true;
         }
         managementStructures.value.queryResult = structureTypeRequest.data.data[i].query;
         managementStructures.value.conexionTextResult = structureTypeRequest.data.data[i]._id;
         break;
      }
    }
  }

  public editStructuresTypes() {
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const headerCardFormStructuresManagement = Metadata.getInstance<Tabs>('headerCardFormStructuresManagement');
    const children0 = managementStructures.tabs[0].children[0];
  }

  public backToGridButtonHierarchyType() {
    const managementStructuresTab2grid = Metadata.getInstance<TekGrid>('managementStructuresTab2grid');
    const hierarchyTypeDetailTabs = Metadata.getInstance<Tabs>('hierarchyTypeDetailTabs');
    managementStructuresTab2grid.isVisible = true;
    hierarchyTypeDetailTabs.isVisible = false;
  }

  public backToGridButton({ event, row }: any) {
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const headerCardFormStructuresManagement = Metadata.getInstance<Form>('headerCardFormStructuresManagement');
    const managementStructuresMasterDetailTab1FormResult = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    managementStructures.isVisible = false;
    managementStructuresTab1grid.isVisible = true;
    headerCardFormStructuresManagement.isVisible = false;// isVisible False, icone (seta) voltar.
  }

  public backButtonStructuresTypes(disabled: any) {
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const children0 = managementStructures.tabs[0].children[0];
    if (children0.children) {
      for (let i = 0; i < children0.children.length; i += 1) {
        if (disabled === true) {
          children0.children[i].disabled = disabled;
          children0.children[6].isVisible = false;
        } else {
          children0.children[i].disabled = disabled;
          children0.children[6].isVisible = true;
        }
      }
    }
  }

  public async addNewRowStructureTypeModal() {
    this.addStructureTypeModal = await ModalService.createFromJson('ManagementStructures/Modal/structureTypeAddModal', true);
    this.addStructureTypeModal.show();
  }

  public structureTypeCloseModal() {
    this.addStructureTypeModal?.destroy();
    this.addStructureTypeModal.hide();
  }

  public messageSalveSuccess() {
    AlertService.show({
            name: 'custom-alert',
            text: "Salvo com Sucesso",
            right: true,
            top: true,
            color: "success"
        });
  }

  public saveHierarchTypeTab() {
    const hierarchyTypeTabForm = Metadata.getInstance<Form>('hierarchyTypeFormTab1');
    hierarchyTypeTabForm.children.forEach(function (formTabHierarchyType) {
    formTabHierarchyType.disabled = true;
    });
    hierarchyTypeTabForm.children[5].isVisible = false;// Habilitando o botão salvar tab hierarchyTypeTabForm
  }

  public async saveAddHierarchType() {
    const form = Metadata.getInstance<Form>('structureTypeForm');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    let isActiveCheckBox = '';
    if (form.value.isActive === true) {
      isActiveCheckBox = 'Y';
    } else {
      isActiveCheckBox = 'N';
    }
     let data: any = {};
     data = {dataset: [{changed: false, descriptionField: form.value.descriptionField, fieldKey: form.value.fieldKey, isActive: isActiveCheckBox, connection: form.value.connectionName, loadFields: form.value.loadFields, name: form.value.name, query: form.value.queryAdd, __createdLocal: true, __is_new: true, disableLoader: false}], disableLoader: false, requestType: "DataSet"};
     LoadingService.show();
     await Http.post('/structureType/save', data);
     LoadingService.hide();
     this.messageSalveSuccess();
     this.structureTypeCloseModal();
     await managementStructuresTab1grid.reload();
  }

  public async saveButtonStructuresTypes({row}: any) {
    await this.backButtonStructuresTypes(true);
    const managementStructures = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const uriSaveStructures = '/structureType/save';
    let isActiveCheckBox = '';
    if (managementStructures.value.activeCheckBox === true) {
      isActiveCheckBox = 'Y';
    } else {
      isActiveCheckBox = 'N';
    }
    let data = {};

      data = {dataset: [{_id: managementStructures.value._id, changed: true, descriptionField: managementStructures.value.descriptionField, fieldKey: managementStructures.value.keyAreaTextResult, isActive: isActiveCheckBox, connection: managementStructures.value.connection, loadFields: managementStructures.value.descriptionFieldtextResult, name: managementStructures.value.nameTextResult, query: managementStructures.value.query, __createdLocal: true, __is_new: true, disableLoader: false}], disableLoader: false, requestType: "DataSet"};

    await Http.post(uriSaveStructures, data);
    this.messageSalveSuccess();
    LoadingService.show();
    await managementStructuresTab1grid.reload();
    LoadingService.hide();
    await this.backButtonStructuresTypes(false);
  }

  public async findNames() {
     LoadingService.show();
     const uriStructure = '/structureNext';
     const uriZmartOperator = '/zmartOperator';
     const requestStructure = await Http.get(uriStructure);
     const dataValues = [];
     for (let i = 0; i < requestStructure.data.data.length; i += 1) {
        const request = {filter: [{operator: "IN", name: "CDOPERADOR", value: requestStructure.data.data[i].usersResponsible}], page: 1, requestType: "FilterData"};
        const data = Http.post(uriZmartOperator, request);
        dataValues.push(data);
     }
     const request = (await Promise.all(dataValues));
     const dataObject: any = {};
     for (let i = 0; i < request.length; i += 1) {
       if (request[i].data?.dataset?.zmartOperator && request[i].data.dataset.zmartOperator.length > 0) {
          for (let j = 0; j < request[i].data.dataset.zmartOperator.length; j += 1) {
            dataObject[request[i].data.dataset.zmartOperator[j].CDOPERADOR] = request[i].data.dataset.zmartOperator[j].NMOPERADOR;
          }
        }
      }
      this.requestNames = dataObject;
      this.findAccessProfiles();
      window.setTimeout(() => LoadingService.hide(), 2500);
  }

  public async findAccessProfiles() {
    const dataObject: any = {};
    const uriZmartAccessProfile = '/zmartAccessProfile';
    const request = await Http.get(uriZmartAccessProfile);
    for (let i = 0; i < request.data.data.length; i += 1) {
      dataObject[request.data.data[i].NRPERFILACESSO] = request.data.data[i].NMPERFILACESSO;
    }
    this.requestAccessProfiles = dataObject;
  }

  public async selectedCheckboxRows({ component, event }: any) {
    event?.preventDefault();
    this.row = component.selectedRows;
    this.showMessageDeleteManagementStructure();
  }

  private back: any = {
    click: () => DialogService.hide(),
  };

  public showMessageDeleteManagementStructure() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação da operação',
      text: 'Deseja excluir esta(s) linha(s)?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteManagementStructure(); DialogService.hide(); } }, outline: true },
        { name: 'no', component: 'ZdButton', label: 'Não', events: this.back as any },
      ],
    });
  }

  public async deleteManagementStructure() {
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const uriDeleteManagementStructure = '/structureType/remove';
    let request = {};
    const results = [];
    for (let i = 0; i < managementStructuresTab1grid.selectedRows.length; i += 1) {
      request = {dataset: [{_id: managementStructuresTab1grid.selectedRows[i]._id}], disableLoader: false, origin: {containerLabel: "Structure", containerName: "structure", widgetLabel: "Structure Types", widgetName: "structure_types"}, requestType: "DataSet"};
      results.push(Http.post(uriDeleteManagementStructure, request));
    }
    await Promise.all(results);
    await managementStructuresTab1grid.reload();
  }

  /* public addNewManagementStructures() {
    this.openManagementStructuresAddModal();
  }

  public async createManagementStructuresAddResponsibleUserModal() {
    if (!this.managementStructuresAddResponsibleUserModal) {
       this.managementStructuresAddResponsibleUserModal = await ModalService.createFromJson('ManagementStructures/Modal/managementStructuresAddResponsibleUserModal', true);
    }
  }

  public openManagementStructuresAddResponsibleUserModal() {
    this.managementStructuresAddResponsibleUserModal.show();
  }

  public closeManagementStructuresAddResponsibleUserModal() {
    this.managementStructuresAddResponsibleUserModal.hide();
  }

  public async addUsersForUserResponsible() {
    const managementStructuresAddResponsibleUserModalTekGrid = Metadata.getInstance<TekGrid>('managementStructuresAddResponsibleUserModal-tek-grid');
    const select = Metadata.getInstance<SelectMultiple>('responsibleUserAutoComplete');
    const data = await select.datasource.updateData(managementStructuresAddResponsibleUserModalTekGrid.selectedRows);
    select.setValue(data);
    this.closeManagementStructuresAddResponsibleUserModal();
  }

  public async createManagementStructuresAddAccessProfilesModal() {
    if (!this.managementStructuresAddAccessProfilesModal) {
       this.managementStructuresAddAccessProfilesModal = await ModalService.createFromJson('ManagementStructures/Modal/managementStructuresAddAccessProfilesModal', true);
    }
  }

  public openManagementStructuresAddAccessProfilesModal() {
    this.managementStructuresAddAccessProfilesModal.show();
  }

  public closeManagementStructuresAddAccessProfilesModal() {
    this.managementStructuresAddAccessProfilesModal.hide();
  }

   public async addAccessProfilesForAccessProfiles() {
    const managementStructuresAddAccessProfiles = Metadata.getInstance<TekGrid>('managementStructuresAddAccessProfilesModal-tek-grid');
    const select = Metadata.getInstance<SelectMultiple>('accessProfilesAutoComplete');
    const data = await select.datasource.updateData(managementStructuresAddAccessProfiles.selectedRows);
    select.setValue(data);
    this.closeManagementStructuresAddAccessProfilesModal();
  } */

}
